@@ -1,5 +1,8 @@
 # -*- coding: utf-8 -*-
 #
+# Like what you see? Join us!
+# https://www.univention.com/about-us/careers/vacancies/
+#
 # Copyright 2004-2022 Univention GmbH
 #
 # https://www.univention.de/
@@ -35,9 +38,7 @@ from __future__ import absolute_import
 
 import base64
 import calendar
-import codecs
 import copy
-import hashlib
 import os
 import re
 import time
@@ -743,6 +744,20 @@ property_descriptions = {
 		show_in_lists=False,
 		cli_enabled=False,
 	),
+	'univentionObjectIdentifier': univention.admin.property(
+		short_description=_('Immutable Object Identifier'),
+		long_description=_('Immutable attribute to track the identity of an object in UDM'),
+		syntax=univention.admin.syntax.string,
+		may_change=False,
+		dontsearch=True,
+	),
+	'univentionSourceIAM': univention.admin.property(
+		short_description=_('Immutable Identifier of the source IAM'),
+		long_description=_('Immutable attribute to identfy source IAM'),
+		syntax=univention.admin.syntax.string,
+		may_change=False,
+		dontsearch=True,
+	),
 }
 
 default_property_descriptions = copy.deepcopy(property_descriptions)  # for later reset of descriptions
@@ -1330,6 +1345,8 @@ mapping.register('jpegPhoto', 'jpegPhoto', univention.admin.mapping.mapBase64, u
 mapping.register('umcProperty', 'univentionUMCProperty', mapKeyAndValue, unmapKeyAndValue)
 mapping.register('lockedTime', 'sambaBadPasswordTime', mapWindowsFiletime, unmapWindowsFiletime)
 mapping.register('accountActivationDate', 'krb5ValidStart', mapDateTimeTimezoneTupleToUTCDateTimeString, unmapUTCDateTimeToLocaltime, encoding='ASCII')
+mapping.register('univentionObjectIdentifier', 'univentionObjectIdentifier', None, univention.admin.mapping.ListToString)
+mapping.register('univentionSourceIAM', 'univentionSourceIAM', None, univention.admin.mapping.ListToString)
 
 mapping.registerUnmapping('sambaRID', unmapSambaRid)
 mapping.registerUnmapping('passwordexpiry', unmapPasswordExpiry)
@@ -1689,6 +1706,11 @@ class object(univention.admin.handlers.simpleLdap):
 		else:
 			self['uidNumber'] = self.request_lock('uidNumber')
 
+		if self['univentionObjectIdentifier']:
+			univention.admin.allocators.acquireUnique(self.lo, self.position, 'univentionObjectIdentifier', self['univentionObjectIdentifier'], 'univentionObjectIdentifier', scope='base')
+			# "False" ==> do not update univentionLastUsedValue in LDAP if a specific value has been specified
+			self.alloc.append(('univentionObjectIdentifier', self['univentionObjectIdentifier'], False))
+
 		self._check_uid_gid_uniqueness()
 
 	def _ldap_pre_ready(self):
@@ -1726,6 +1748,16 @@ class object(univention.admin.handlers.simpleLdap):
 				except univention.admin.uexceptions.noLock:
 					raise univention.admin.uexceptions.mailAddressUsed(self['mailPrimaryAddress'])
 
+		# get lock for mailAlternativeAddress
+		if (not self.exists() or self.hasChanged('mailAlternativeAddress')) and self['mailAlternativeAddress']:
+			old_maas, new_maas = ({addr.lower() for addr in info.get('mailAlternativeAddress', [])} for info in (self.oldinfo, self.info))
+			for added_maa in (new_maas - old_maas):
+				# uniqueness for mailAlternativeAddress
+				try:
+					self.request_lock('mailAlternativeAddress', added_maa)
+				except univention.admin.uexceptions.noLock:
+					raise univention.admin.uexceptions.mailAddressUsed(added_maa)
+
 		if self['unlock'] == '1':
 			self['locked'] = u'0'
 		if self.hasChanged('disabled') and self['disabled'] == '0' and not self.hasChanged('accountActivationDate'):
@@ -1898,11 +1930,6 @@ class object(univention.admin.handlers.simpleLdap):
 		password_nt, password_lm = password_nt.encode('ASCII'), password_lm.encode('ASCII')
 		ml.append(('sambaNTPassword', self.oldattr.get('sambaNTPassword', [b''])[0], password_nt))
 		ml.append(('sambaLMPassword', self.oldattr.get('sambaLMPassword', [b''])[0], password_lm))
-
-		if pwhistoryPolicy.pwhistoryLength is not None:
-			smbpwhistory = self.oldattr.get('sambaPasswordHistory', [b''])[0].decode('ASCII')
-			newsmbPWHistory = self._get_samba_password_history(password_nt, smbpwhistory, pwhistoryPolicy.pwhistoryLength)
-			ml.append(('sambaPasswordHistory', self.oldattr.get('sambaPasswordHistory', [b''])[0], newsmbPWHistory.encode('ASCII')))
 		return ml
 
 	def _modlist_kerberos_password(self, ml):
@@ -2260,43 +2287,6 @@ class object(univention.admin.handlers.simpleLdap):
 
 		return dn
 
-	@classmethod
-	def _get_samba_password_history(cls, newpassword, smbpwhistory, smbpwhlen):
-		"""Get history of previously used passwords.
-
-#		>>> object._get_samba_password_history('186CB09181E2C2ECAAC768C47C729904', 'A047EE4A9DB8BC8B4F3F8A03D72DEB80', 0)
-#		...
-#		>>> object._get_samba_password_history('186CB09181E2C2ECAAC768C47C729904', '', 1)
-#		...
-#		>>> object._get_samba_password_history('186CB09181E2C2ECAAC768C47C729904', 'A047EE4A9DB8BC8B4F3F8A03D72DEB80', 1)
-#		...
-#		>>> object._get_samba_password_history('186CB09181E2C2ECAAC768C47C729904', 'A047EE4A9DB8BC8B4F3F8A03D72DEB80', 2)
-#		...
-		"""
-
-		# calculate the password hash & salt
-		# in binary for calculating the md5:
-		salt = os.urandom(16)
-		# we have to have that in hex:
-		hexsalt = codecs.encode(salt, 'hex').upper().decode('ASCII')
-		# we need the ntpwd binary data to
-		pwd = codecs.decode(newpassword, 'hex')
-		# calculating hash. stored as a 32byte hex in sambaPasswordHistory,
-		# syntax like that: [Salt][MD5(Salt+Hash)]
-		#	First 16bytes ^		^ last 16bytes.
-		pwdhash = hashlib.md5(salt + pwd).hexdigest().upper()
-		smbpwhash = hexsalt + pwdhash
-
-		# split the history
-		pwlist = smbpwhistory.strip().split(' ')
-		# append new hash
-		pwlist.append(smbpwhash)
-		# strip old hashes
-		pwlist = pwlist[-smbpwhlen:]
-		# build history
-		smbpwhistory = ''.join(pwlist)
-		return smbpwhistory
-
 	def __allocate_rid(self, rid):
 		searchResult = self.lo.search(filter='objectClass=sambaDomain', attr=['sambaSID'])
 		domainsid = searchResult[0][1]['sambaSID'][0]
@@ -2366,7 +2356,7 @@ class object(univention.admin.handlers.simpleLdap):
 				filter.transform_to_conjunction(univention.admin.filter.parse(u'(&(shadowExpire=1)(|(sambaAcctFlags=[UD       ])(sambaAcctFlags==[ULD       ])))'))
 			elif filter.value == u'posix_kerberos':
 				filter.transform_to_conjunction(univention.admin.filter.parse(u'(&(shadowExpire=1)(krb5KDCFlags=254))'))
-			elif filter.value == u'*':
+			elif filter.operator == u'=*':
 				filter.variable = u'uid'
 		elif filter.variable == 'userexpiry':
 			try:
