@@ -1,5 +1,8 @@
 # -*- coding: utf-8 -*-
 #
+# Like what you see? Join us!
+# https://www.univention.com/about-us/careers/vacancies/
+#
 # Copyright 2004-2022 Univention GmbH
 #
 # https://www.univention.de/
@@ -36,6 +39,8 @@ from __future__ import absolute_import
 import re
 import bcrypt
 import hashlib
+import codecs
+import os
 from typing import List, Optional, Tuple  # noqa: F401
 
 import heimdal
@@ -256,7 +261,9 @@ def get_password_history(password, pwhistory, pwhlen):
 	2
 	"""
 	# create hash
-	if configRegistry.is_true('password/hashing/bcrypt'):
+	if password.startswith('{NT}'):
+		newpwhash = password
+	elif configRegistry.is_true('password/hashing/bcrypt'):
 		newpwhash = "{BCRYPT}%s" % (bcrypt_hash(password))
 	else:
 		newpwhash = crypt(password)
@@ -298,6 +305,10 @@ def password_already_used(password, pwhistory):
 				password_hash = line[len('{BCRYPT}'):]
 				if bcrypt.checkpw(password.encode('utf-8'), password_hash.encode('ASCII')):
 					return True
+			elif linesplit[0] == '{NT}':
+				password_hash = line[len('{NT}$'):]
+				if password_hash == ntlm(password)[0]:
+					return True
 			else:
 				password_hash = crypt(password, linesplit[1], linesplit[2])
 		except IndexError:  # old style password history entry, no method id/salt in there
